//========================================================================
//
// OPVPSplash.h
//
//========================================================================

#ifndef OPVPSPLASH_H
#define OPVPSPLASH_H

#include <aconf.h>

#ifdef USE_GCC_PRAGMAS
#pragma interface
#endif

#include <typeinfo>
#include "SplashTypes.h"
#include "SplashPath.h"
#include "SplashPattern.h"
#include "OPVPWrapper.h"
#include "SplashErrorCodes.h"

/* extra error code */
#define splashErrOPVP 100

#define OPVP_MAX_CLIPPATH_LENGTH 2000

class SplashBitmap;
class SplashGlyphBitmap;
class SplashState;
class SplashPattern;
class SplashScreen;
class SplashPath;
class SplashXPath;
class SplashClip;
class SplashFont;

class OPVPClipPath {
public:
  OPVPClipPath(SplashPath *pathA, GBool eoA);
  void push();
  static OPVPClipPath *pop();
  ~OPVPClipPath() { delete path; }
  SplashPath *getPath() { return path; }
  GBool getEo() { return eo; }
  GBool getSaved() { return saved; }
private:
  OPVPClipPath *copy();
  OPVPClipPath *next;
  SplashPath *path;
  GBool eo;
  GBool saved;
  static OPVPClipPath *stackTop;
};

//------------------------------------------------------------------------

// Retrieves the next pixel in an image mask.  Normally, fills in
// *<pixel> and returns true.  If the image stream is exhausted,
// returns false.
typedef GBool (*SplashImageMaskSource)(void *data, SplashMono1 *pixel);

// Retrieves the next pixel in an image.  Normally, fills in *<pixel>
// (pixel color) and *<alpha> (1 for opaque, 0 for transparent), and
// returns true.  If the image stream is exhausted, returns false.
typedef GBool (*SplashImageSource)(void *data, SplashColor *pixel,
				   Guchar *alpha);

//------------------------------------------------------------------------
// Splash
//------------------------------------------------------------------------

class OPVPSplash {
public:

  // Create a new rasterizer object.
  OPVPSplash(OPVP_api_procs *opvpA, int printerContextA,
    int nOptions, char *optionKeys[], char *optionVals[]);

  virtual ~OPVPSplash();

  //----- state read

  SplashPattern *getStrokePattern();
  SplashPattern *getFillPattern();
  SplashScreen *getScreen();
  SplashCoord getLineWidth();
  int getLineCap();
  int getLineJoin();
  SplashCoord getMiterLimit();
  SplashCoord getFlatness();
  SplashCoord *getLineDash();
  int getLineDashLength();
  SplashCoord getLineDashPhase();
  SplashClip *getClip();

  //----- state write

  void setStrokePattern(SplashPattern *strokeColor);
  void setFillPattern(SplashPattern *fillColor);
  void setScreen(SplashScreen *screen);
  void setLineWidth(SplashCoord lineWidth);
  void setLineCap(int lineCap);
  void setLineJoin(int lineJoin);
  void setMiterLimit(SplashCoord miterLimit);
  void setFlatness(SplashCoord flatness);
  // the <lineDash> array will be copied
  void setLineDash(SplashCoord *lineDash, int lineDashLength,
		   SplashCoord lineDashPhase);
  void clipResetToRect(SplashCoord x0, SplashCoord y0,
		       SplashCoord x1, SplashCoord y1);
  SplashError clipToPath(SplashPath *path, GBool eo);

  //----- state save/restore

  void saveState();
  SplashError restoreState();

  //----- drawing operations

  // Fill the bitmap with <color>.  This is not subject to clipping.
  void clear(SplashColor color);

  // Stroke a path using the current stroke pattern.
  SplashError stroke(SplashPath *path);

  // Fill a path using the current fill pattern.
  SplashError fill(SplashPath *path, GBool eo);

  // Draw a character, using the current fill pattern.
  SplashError fillChar(SplashCoord x, SplashCoord y, int c,
    SplashFont *font, Unicode *u, double *fontMat);

  // Draw a glyph, using the current fill pattern.  This function does
  // not free any data, i.e., it ignores glyph->freeData.
  // not used in vector mode
  SplashError fillGlyph(SplashCoord x, SplashCoord y,
			SplashGlyphBitmap *glyph);

  // Draws an image mask using the fill color.  This will read <w>*<h>
  // pixels from <src>, in raster order, starting with the top line.
  // "1" pixels will be drawn with the current fill color; "0" pixels
  // are transparent.  The matrix:
  //    [ mat[0] mat[1] 0 ]
  //    [ mat[2] mat[3] 0 ]
  //    [ mat[4] mat[5] 1 ]
  // maps a unit square to the desired destination for the image, in
  // PostScript style:
  //    [x' y' 1] = [x y 1] * mat
  // Note that the Splash y axis points downward, and the image source
  // is assumed to produce pixels in raster order, starting from the
  // top line.
  SplashError fillImageMask(SplashImageMaskSource src, void *srcData,
			    int w, int h, SplashCoord *mat);

  // Draw an image.  This will read <w>*<h> pixels from <src>, in
  // raster order, starting with the top line.  These pixels are
  // assumed to be in the source mode, <srcMode>.  The following
  // combinations of source and target modes are supported:
  //    source       target
  //    ------       ------
  //    Mono1        Mono1
  //    Mono8        Mono1   -- with dithering
  //    Mono8        Mono8
  //    RGB8         RGB8
  //    BGR8packed   BGR8Packed
  // The matrix behaves as for fillImageMask.
  SplashError drawImage(SplashImageSource src, void *srcData,
			SplashColorMode srcMode,
			int w, int h, SplashCoord *mat,
			GBool mask);

  //~ drawMaskedImage

  //----- misc

  // Return the associated bitmap.
  SplashBitmap *getBitmap() { return NULL; }

  // Toggle debug mode on or off.
  void setDebugMode(GBool debugModeA) { debugMode = debugModeA; }

  void setColorMode(int colorModeA);
  void setStateBypass(GBool bypass) {stateBypass = bypass;}
  void endPage();

private:
  SplashError makePath(SplashPath *path);
  void makeBrush(SplashPattern *pattern, OPVP_Brush *brush);
  SplashError doClipPath(SplashPath *path, GBool eo);
  OPVP_ColorSpace getOPVPColorSpace();
  GBool equalPattern(SplashPattern *pt1, SplashPattern *pt2);
  GBool isRectanglePath(SplashPath *path, SplashCoord *xMin,
    SplashCoord *yMin, SplashCoord *xMax, SplashCoord *yMax);
  SplashError makeRectanglePath(SplashCoord x0, SplashCoord y0,
    SplashCoord x1, SplashCoord y1, SplashPath **p);
  SplashError drawImageFastWithCTM(SplashImageSource src, void *srcData,
			      int w, int h, int tx, int ty,
			      SplashCoord *mat);
  SplashError drawImageNotShear(SplashImageSource src,
                              void *srcData,
			      int w, int h,
			      int tx, int ty,
			      int scaledWidth, int scaledHeight,
			      int xSign, int ySign, GBool rot);
  SplashError fillImageMaskFastWithCTM(SplashImageMaskSource src,
       void *srcData, int w, int h, int tx, int ty,SplashCoord *mat);
  void OPVPSplash::arcToCurve(SplashCoord x0, SplashCoord y0,
    SplashCoord x3, SplashCoord y3,
    SplashCoord cx, SplashCoord cy, SplashCoord *rx1, SplashCoord *ry1,
    SplashCoord *rx2, SplashCoord *ry2);

  SplashError strokeByMyself(SplashPath *path);
  SplashError fillByMyself(SplashPath *path, GBool eo);
  void strokeNarrow(SplashXPath *xPath);
  void strokeWide(SplashXPath *xPath);
  SplashXPath *makeDashedPath(SplashXPath *xPath);
  void drawSpan(int x0, int x1, int y, GBool noClip);
#ifdef OLD_DRAW_IMAGE
  void drawPixel(int x, int y, SplashColor *color, GBool noClip);
#endif
  void drawPixel(int x, int y, GBool noClip);

  char *getOption(char *key, int nOptions, char *optionKeys[],
    char *optionVals[]);

  OPVPWrapper *opvp;
  int printerContext;

  SplashState *state;
  GBool debugMode;
  int colorMode;
  GBool stateBypass;
  OPVPClipPath *clipPath;

  GBool oldLipsDriver;
  GBool clipPathNotSaved;
  GBool noShearImage;
  GBool noLineStyle;
  GBool noClipPath;
  GBool savedNoClipPath;
  GBool noBitmapChar;
};

#endif
