//========================================================================
//
// TSplash.h
//
//========================================================================

#ifndef TSPLASH_H
#define TSPLASH_H

#include <aconf.h>

#ifdef USE_GCC_PRAGMAS
#pragma interface
#endif

#include "SplashTypes.h"

class SplashBitmap;
class SplashGlyphBitmap;
class SplashState;
class SplashPattern;
class SplashScreen;
class SplashPath;
class SplashXPath;
class SplashClip;
class SplashFont;

//------------------------------------------------------------------------

// Retrieves the next pixel in an image mask.  Normally, fills in
// *<pixel> and returns true.  If the image stream is exhausted,
// returns false.
typedef GBool (*SplashImageMaskSource)(void *data, SplashMono1 *pixel);

// Retrieves the next pixel in an image.  Normally, fills in *<pixel>
// (pixel color) and *<alpha> (1 for opaque, 0 for transparent), and
// returns true.  If the image stream is exhausted, returns false.
typedef GBool (*SplashImageSource)(void *data, SplashColor *pixel,
				   Guchar *alpha);


//------------------------------------------------------------------------
// TSplash
//------------------------------------------------------------------------

class TSplash {
public:

  //----- state read

  virtual SplashPattern *getStrokePattern() = 0;
  virtual SplashPattern *getFillPattern() = 0;
  virtual SplashScreen *getScreen() = 0;
  virtual SplashCoord getLineWidth() = 0;
  virtual int getLineCap() = 0;
  virtual int getLineJoin() = 0;
  virtual SplashCoord getMiterLimit() = 0;
  virtual SplashCoord getFlatness() = 0;
  virtual SplashCoord *getLineDash() = 0;
  virtual int getLineDashLength() = 0;
  virtual SplashCoord getLineDashPhase() = 0;
  virtual SplashClip *getClip() = 0;

  //----- state write

  virtual void setStrokePattern(SplashPattern *strokeColor) = 0;
  virtual void setFillPattern(SplashPattern *fillColor) = 0;
  virtual void setScreen(SplashScreen *screen) = 0;
  virtual void setLineWidth(SplashCoord lineWidth) = 0;
  virtual void setLineCap(int lineCap) = 0;
  virtual void setLineJoin(int lineJoin) = 0;
  virtual void setMiterLimit(SplashCoord miterLimit) = 0;
  virtual void setFlatness(SplashCoord flatness) = 0;
  // the <lineDash> array will be copied
  virtual void setLineDash(SplashCoord *lineDash, int lineDashLength,
		   SplashCoord lineDashPhase) = 0;
  virtual void clipResetToRect(SplashCoord x0, SplashCoord y0,
		       SplashCoord x1, SplashCoord y1) = 0;
  virtual SplashError clipToRect(SplashCoord x0, SplashCoord y0,
			 SplashCoord x1, SplashCoord y1) = 0;
  virtual SplashError clipToPath(SplashPath *path, GBool eo) = 0;

  //----- state save/restore

  virtual void saveState() = 0;
  virtual SplashError restoreState() = 0;

  //----- drawing operations

  // Fill the bitmap with <color>.  This is not subject to clipping.
  virtual void clear(SplashColor color) = 0;

  // Stroke a path using the current stroke pattern.
  virtual SplashError stroke(SplashPath *path) = 0;

  // Fill a path using the current fill pattern.
  virtual SplashError fill(SplashPath *path, GBool eo) = 0;

  // Fill a path, XORing with the current fill pattern.
  virtual SplashError xorFill(SplashPath *path, GBool eo) = 0;

  // Draw a character, using the current fill pattern.
  virtual SplashError fillChar(SplashCoord x, SplashCoord y, int c, SplashFont *font, Unicode *u) = 0;

  // Draw a glyph, using the current fill pattern.  This function does
  // not free any data, i.e., it ignores glyph->freeData.
  virtual SplashError fillGlyph(SplashCoord x, SplashCoord y,
			SplashGlyphBitmap *glyph) = 0;

  // Draws an image mask using the fill color.  This will read <w>*<h>
  // pixels from <src>, in raster order, starting with the top line.
  // "1" pixels will be drawn with the current fill color; "0" pixels
  // are transparent.  The matrix:
  //    [ mat[0] mat[1] 0 ]
  //    [ mat[2] mat[3] 0 ]
  //    [ mat[4] mat[5] 1 ]
  // maps a unit square to the desired destination for the image, in
  // PostScript style:
  //    [x' y' 1] = [x y 1] * mat
  // Note that the Splash y axis points downward, and the image source
  // is assumed to produce pixels in raster order, starting from the
  // top line.
  virtual SplashError fillImageMask(SplashImageMaskSource src, void *srcData,
			    int w, int h, SplashCoord *mat) = 0;

  // Draw an image.  This will read <w>*<h> pixels from <src>, in
  // raster order, starting with the top line.  These pixels are
  // assumed to be in the source mode, <srcMode>.  The following
  // combinations of source and target modes are supported:
  //    source       target
  //    ------       ------
  //    Mono1        Mono1
  //    Mono8        Mono1   -- with dithering
  //    Mono8        Mono8
  //    RGB8         RGB8
  //    BGR8packed   BGR8Packed
  // The matrix behaves as for fillImageMask.
  virtual SplashError drawImage(SplashImageSource src, void *srcData,
			SplashColorMode srcMode,
			int w, int h, SplashCoord *mat) = 0;

  //~ drawMaskedImage

  //----- misc

  // Return the associated bitmap.
  virtual SplashBitmap *getBitmap() = 0;

  // Toggle debug mode on or off.
  virtual void setDebugMode(GBool debugModeA) = 0;
};

#endif
